"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const WarrantModule_1 = __importDefault(require("./WarrantModule"));
const WarrantClient_1 = __importDefault(require("../WarrantClient"));
const ObjectType_1 = require("../types/ObjectType");
class Feature {
    constructor(featureId) {
        this.featureId = featureId;
    }
    //
    // Static methods
    //
    static create(feature, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.post({
                    url: "/v1/features",
                    data: feature,
                    options,
                });
                return new Feature(response.featureId);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static get(featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: `/v1/features/${featureId}`,
                    options,
                });
                return new Feature(response.featureId);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static delete(featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield WarrantClient_1.default.httpClient.delete({
                    url: `/v1/features/${featureId}`,
                    options,
                });
            }
            catch (e) {
                throw e;
            }
        });
    }
    static listFeatures(listOptions = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: "/v1/features",
                    params: listOptions,
                    options,
                });
                return response.map((feature) => new Feature(feature.featureId));
            }
            catch (e) {
                throw e;
            }
        });
    }
    static listFeaturesForPricingTier(pricingTierId, listOptions = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: `/v1/pricing-tiers/${pricingTierId}/features`,
                    params: listOptions,
                    options,
                });
                return response.map((feature) => new Feature(feature.featureId));
            }
            catch (e) {
                throw e;
            }
        });
    }
    static assignFeatureToPricingTier(pricingTierId, featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.create({
                object: {
                    objectType: ObjectType_1.ObjectType.Feature,
                    objectId: featureId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.PricingTier,
                    objectId: pricingTierId,
                }
            }, options);
        });
    }
    static removeFeatureFromPricingTier(pricingTierId, featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.delete({
                object: {
                    objectType: ObjectType_1.ObjectType.Feature,
                    objectId: featureId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.PricingTier,
                    objectId: pricingTierId,
                }
            }, options);
        });
    }
    static listFeaturesForTenant(tenantId, listOptions = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: `/v1/tenants/${tenantId}/features`,
                    params: listOptions,
                    options,
                });
                return response.map((feature) => new Feature(feature.featureId));
            }
            catch (e) {
                throw e;
            }
        });
    }
    static assignFeatureToTenant(tenantId, featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.create({
                object: {
                    objectType: ObjectType_1.ObjectType.Feature,
                    objectId: featureId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.Tenant,
                    objectId: tenantId,
                }
            }, options);
        });
    }
    static removeFeatureFromTenant(tenantId, featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.delete({
                object: {
                    objectType: ObjectType_1.ObjectType.Feature,
                    objectId: featureId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.Tenant,
                    objectId: tenantId,
                }
            }, options);
        });
    }
    static listFeaturesForUser(userId, listOptions = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: `/v1/users/${userId}/features`,
                    params: listOptions,
                    options,
                });
                return response.map((feature) => new Feature(feature.featureId));
            }
            catch (e) {
                throw e;
            }
        });
    }
    static assignFeatureToUser(userId, featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.create({
                object: {
                    objectType: ObjectType_1.ObjectType.Feature,
                    objectId: featureId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.User,
                    objectId: userId,
                }
            }, options);
        });
    }
    static removeFeatureFromUser(userId, featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.delete({
                object: {
                    objectType: ObjectType_1.ObjectType.Feature,
                    objectId: featureId,
                },
                relation: "member",
                subject: {
                    objectType: ObjectType_1.ObjectType.User,
                    objectId: userId,
                }
            }, options);
        });
    }
    // WarrantObject methods
    getObjectType() {
        return ObjectType_1.ObjectType.Feature;
    }
    getObjectId() {
        return this.featureId;
    }
}
exports.default = Feature;
