"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Authorization_1 = __importDefault(require("./Authorization"));
const Feature_1 = __importDefault(require("./Feature"));
const PricingTier_1 = __importDefault(require("./PricingTier"));
const User_1 = __importDefault(require("./User"));
const WarrantClient_1 = __importDefault(require("../WarrantClient"));
const ObjectType_1 = require("../types/ObjectType");
class Tenant {
    constructor(tenantId, name) {
        this.tenantId = tenantId;
        this.name = name;
    }
    //
    // Static methods
    //
    static create(tenant = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.post({
                    url: "/v1/tenants",
                    data: tenant,
                    options,
                });
                return new Tenant(response.tenantId, response.name);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static batchCreate(tenants, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.post({
                    url: "/v1/tenants",
                    data: tenants,
                    options,
                });
                return response.map((tenant) => new Tenant(tenant.tenantId, tenant.name));
            }
            catch (e) {
                throw e;
            }
        });
    }
    static get(tenantId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: `/v1/tenants/${tenantId}`,
                    options,
                });
                return new Tenant(response.tenantId, response.name);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static update(tenantId, tenant, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.put({
                    url: `/v1/tenants/${tenantId}`,
                    data: tenant,
                    options,
                });
                return new Tenant(response.tenantId, response.name);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static delete(tenantId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield WarrantClient_1.default.httpClient.delete({
                    url: `/v1/tenants/${tenantId}`,
                    options,
                });
            }
            catch (e) {
                throw e;
            }
        });
    }
    static listTenants(listOptions = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: "/v1/tenants",
                    params: listOptions,
                    options,
                });
                return response.map((tenant) => new Tenant(tenant.tenantId, tenant.name));
            }
            catch (e) {
                throw e;
            }
        });
    }
    static listTenantsForUser(userId, listOptions = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: `/v1/users/${userId}/tenants`,
                    params: listOptions,
                    options,
                });
                return response.map((tenant) => new Tenant(tenant.tenantId, tenant.name));
            }
            catch (e) {
                throw e;
            }
        });
    }
    //
    // Instance methods
    //
    listUsers(listOptions = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return User_1.default.listUsersForTenant(this.tenantId, listOptions, options);
        });
    }
    assignUser(userId, role, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return User_1.default.assignUserToTenant(this.tenantId, userId, role, options);
        });
    }
    removeUser(userId, role, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return User_1.default.removeUserFromTenant(this.tenantId, userId, role, options);
        });
    }
    listPricingTiers(listOptions = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return PricingTier_1.default.listPricingTiersForTenant(this.tenantId, listOptions, options);
        });
    }
    assignPricingTier(pricingTierId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return PricingTier_1.default.assignPricingTierToTenant(this.tenantId, pricingTierId, options);
        });
    }
    removePricingTier(pricingTierId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return PricingTier_1.default.removePricingTierFromTenant(this.tenantId, pricingTierId, options);
        });
    }
    listFeatures(listOptions = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Feature_1.default.listFeaturesForTenant(this.tenantId, listOptions, options);
        });
    }
    assignFeature(featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Feature_1.default.assignFeatureToTenant(this.tenantId, featureId, options);
        });
    }
    removeFeature(featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Feature_1.default.removeFeatureFromTenant(this.tenantId, featureId, options);
        });
    }
    hasFeature(featureId, context = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Authorization_1.default.hasFeature({ featureId: featureId, subject: { objectType: ObjectType_1.ObjectType.Tenant, objectId: this.tenantId }, context: context }, options);
        });
    }
    // WarrantObject methods
    getObjectType() {
        return ObjectType_1.ObjectType.Tenant;
    }
    getObjectId() {
        return this.tenantId;
    }
}
exports.default = Tenant;
