import Feature from "./Feature";
import Permission from "./Permission";
import PricingTier from "./PricingTier";
import Role from "./Role";
import Tenant from "./Tenant";
import Warrant from "./WarrantModule";
import { ListFeatureOptions } from "../types/Feature";
import { WarrantRequestOptions } from "../types/WarrantRequestOptions";
import { ListPermissionOptions } from "../types/Permission";
import { ListPricingTierOptions } from "../types/PricingTier";
import { ListRoleOptions } from "../types/Role";
import { CreateUserParams, ListUserOptions, UpdateUserParams } from "../types/User";
import { ListTenantOptions } from "../types/Tenant";
import { PolicyContext, WarrantObject } from "../types/Warrant";
export default class User implements WarrantObject {
    userId: string;
    email?: string;
    constructor(userId: string, email?: string);
    static create(user?: CreateUserParams, options?: WarrantRequestOptions): Promise<User>;
    static batchCreate(users: CreateUserParams[], options?: WarrantRequestOptions): Promise<User[]>;
    static get(userId: string, options?: WarrantRequestOptions): Promise<User>;
    static update(userId: string, user: UpdateUserParams, options?: WarrantRequestOptions): Promise<User>;
    static delete(userId: string, options?: WarrantRequestOptions): Promise<void>;
    static listUsers(listOptions?: ListUserOptions, options?: WarrantRequestOptions): Promise<User[]>;
    static listUsersForTenant(tenantId: string, listOptions?: ListUserOptions, options?: WarrantRequestOptions): Promise<User[]>;
    static assignUserToTenant(tenantId: string, userId: string, role: string, options?: WarrantRequestOptions): Promise<Warrant>;
    static removeUserFromTenant(tenantId: string, userId: string, role: string, options?: WarrantRequestOptions): Promise<void>;
    listTenants(listOptions?: ListTenantOptions, options?: WarrantRequestOptions): Promise<Tenant[]>;
    listRoles(listOptions?: ListRoleOptions, options?: WarrantRequestOptions): Promise<Role[]>;
    assignRole(roleId: string, options?: WarrantRequestOptions): Promise<Warrant>;
    removeRole(roleId: string, options?: WarrantRequestOptions): Promise<void>;
    listPermissions(listOptions?: ListPermissionOptions, options?: WarrantRequestOptions): Promise<Permission[]>;
    assignPermission(permissionId: string, options?: WarrantRequestOptions): Promise<Warrant>;
    removePermission(permissionId: string, options?: WarrantRequestOptions): Promise<void>;
    hasPermission(permissionId: string, context?: PolicyContext, options?: WarrantRequestOptions): Promise<boolean>;
    listPricingTiers(listOptions?: ListPricingTierOptions, options?: WarrantRequestOptions): Promise<PricingTier[]>;
    assignPricingTier(pricingTierId: string, options?: WarrantRequestOptions): Promise<Warrant>;
    removePricingTier(pricingTierId: string, options?: WarrantRequestOptions): Promise<void>;
    listFeatures(listOptions?: ListFeatureOptions, options?: WarrantRequestOptions): Promise<Feature[]>;
    assignFeature(featureId: string, options?: WarrantRequestOptions): Promise<Warrant>;
    removeFeature(featureId: string, options?: WarrantRequestOptions): Promise<void>;
    hasFeature(featureId: string, context?: PolicyContext, options?: WarrantRequestOptions): Promise<boolean>;
    getObjectType(): string;
    getObjectId(): string;
}
