"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Authorization_1 = __importDefault(require("./Authorization"));
const Feature_1 = __importDefault(require("./Feature"));
const Permission_1 = __importDefault(require("./Permission"));
const PricingTier_1 = __importDefault(require("./PricingTier"));
const Role_1 = __importDefault(require("./Role"));
const Tenant_1 = __importDefault(require("./Tenant"));
const WarrantClient_1 = __importDefault(require("../WarrantClient"));
const ObjectType_1 = require("../types/ObjectType");
const WarrantModule_1 = __importDefault(require("./WarrantModule"));
class User {
    constructor(userId, email) {
        this.userId = userId;
        this.email = email;
    }
    //
    // Static methods
    //
    static create(user = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.post({
                    url: "/v1/users",
                    data: user,
                    options,
                });
                return new User(response.userId, response.email);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static batchCreate(users, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.post({
                    url: "/v1/users",
                    data: users,
                    options,
                });
                return response.map((user) => new User(user.userId, user.email));
            }
            catch (e) {
                throw e;
            }
        });
    }
    static get(userId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: `/v1/users/${userId}`,
                    options,
                });
                return new User(response.userId, response.email);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static update(userId, user, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.put({
                    url: `/v1/users/${userId}`,
                    data: user,
                    options,
                });
                return new User(response.userId, response.email);
            }
            catch (e) {
                throw e;
            }
        });
    }
    static delete(userId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                return yield WarrantClient_1.default.httpClient.delete({
                    url: `/v1/users/${userId}`,
                    options,
                });
            }
            catch (e) {
                throw e;
            }
        });
    }
    static listUsers(listOptions = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: "/v1/users",
                    params: listOptions,
                    options,
                });
                return response.map((user) => new User(user.userId, user.email));
            }
            catch (e) {
                throw e;
            }
        });
    }
    static listUsersForTenant(tenantId, listOptions = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const response = yield WarrantClient_1.default.httpClient.get({
                    url: `/v1/tenants/${tenantId}/users`,
                    params: listOptions,
                    options,
                });
                return response.map((user) => new User(user.userId, user.email));
            }
            catch (e) {
                throw e;
            }
        });
    }
    static assignUserToTenant(tenantId, userId, role, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.create({
                object: {
                    objectType: ObjectType_1.ObjectType.Tenant,
                    objectId: tenantId,
                },
                relation: role,
                subject: {
                    objectType: ObjectType_1.ObjectType.User,
                    objectId: userId,
                }
            }, options);
        });
    }
    static removeUserFromTenant(tenantId, userId, role, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return WarrantModule_1.default.delete({
                object: {
                    objectType: ObjectType_1.ObjectType.Tenant,
                    objectId: tenantId,
                },
                relation: role,
                subject: {
                    objectType: ObjectType_1.ObjectType.User,
                    objectId: userId,
                }
            }, options);
        });
    }
    //
    // Instance methods
    //
    listTenants(listOptions = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Tenant_1.default.listTenantsForUser(this.userId, listOptions, options);
        });
    }
    listRoles(listOptions = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Role_1.default.listRolesForUser(this.userId, listOptions, options);
        });
    }
    assignRole(roleId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Role_1.default.assignRoleToUser(this.userId, roleId, options);
        });
    }
    removeRole(roleId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Role_1.default.removeRoleFromUser(this.userId, roleId, options);
        });
    }
    listPermissions(listOptions = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Permission_1.default.listPermissionsForUser(this.userId, listOptions, options);
        });
    }
    assignPermission(permissionId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Permission_1.default.assignPermissionToUser(this.userId, permissionId, options);
        });
    }
    removePermission(permissionId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Permission_1.default.removePermissionFromUser(this.userId, permissionId, options);
        });
    }
    hasPermission(permissionId, context = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Authorization_1.default.hasPermission({ permissionId: permissionId, subject: { objectType: ObjectType_1.ObjectType.User, objectId: this.userId }, context: context }, options);
        });
    }
    listPricingTiers(listOptions = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return PricingTier_1.default.listPricingTiersForUser(this.userId, listOptions, options);
        });
    }
    assignPricingTier(pricingTierId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return PricingTier_1.default.assignPricingTierToUser(this.userId, pricingTierId, options);
        });
    }
    removePricingTier(pricingTierId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return PricingTier_1.default.removePricingTierFromUser(this.userId, pricingTierId, options);
        });
    }
    listFeatures(listOptions = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Feature_1.default.listFeaturesForUser(this.userId, listOptions, options);
        });
    }
    assignFeature(featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Feature_1.default.assignFeatureToUser(this.userId, featureId, options);
        });
    }
    removeFeature(featureId, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Feature_1.default.removeFeatureFromUser(this.userId, featureId, options);
        });
    }
    hasFeature(featureId, context = {}, options = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            return Authorization_1.default.hasFeature({ featureId: featureId, subject: { objectType: ObjectType_1.ObjectType.User, objectId: this.userId }, context: context }, options);
        });
    }
    // WarrantObject methods
    getObjectType() {
        return ObjectType_1.ObjectType.User;
    }
    getObjectId() {
        return this.userId;
    }
}
exports.default = User;
